<?php

namespace v1\controllers;

use app\models\Folio;
use common\data\Respuesta;
use common\rest\AuthController;
use v1\models\AclaracionMedia;
use v1\models\BitacoraEstatus;
use v1\models\Estatus;
use v1\models\MediaBitacoraEstatus;
use v1\models\Solicitud;
use v1\models\SolicitudMedia;
use v1\models\SujetoObligado;
use yii\db\Expression;

class AclaracionController extends AuthController {

  public $modelClass = "v1\models\Aclaracion";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $idSolicitud = trim($this->req->get("idSolicitud", ""));
    $buscar = trim($this->req->get("q", ""));

    $query = $this->queryInicial
      ->innerJoin('Solicitud', '{{Solicitud}}.id = {{Aclaracion}}.[[idSolicitud]]');

    if ($id !== "") {
      $query->andWhere(["{{Aclaracion}}.id" => $id]);
    }

    if ($idSolicitud !== "") {
      $query->andWhere(["{{Aclaracion}}.[[idSolicitud]]" => $idSolicitud]);
    }

    if ($buscar) {
      $query->andWhere([
        "OR",
        "f_unaccent({{Solicitud}}.[[leyenda]]) ilike f_unaccent(:q)",
      ])->addParams([':q' => "%{$buscar}%"]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $idSolicitud = trim($this->req->getBodyParam("idSolicitud", ""));
    $valorAclaracion = intval($this->req->getBodyParam('valorAclaracion', ''));
    $archivos = $this->req->getBodyParam("archivos", []);
    $idSolicitaAclaracion = trim($this->req->getBodyParam("idSolicitaAclaracion", ""));
    $idAclarador = trim($this->req->getBodyParam("idAclarador", ""));
    $tipo = trim($this->req->getBodyParam("tipo", ""));

    $usuario = $this->usuario;

    $estatus = Estatus::find()
      ->andWhere(['eliminado' => null]);

    $modelo = null;
    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }

    $tran = \Yii::$app->getDb()->beginTransaction();
    try {
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo->uuid();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }

      $modelo->load($this->req->getBodyParams(), '');

      if ($idSolicitaAclaracion !== "") {
        $modelo->solicitado = new Expression('now()');
      }

      if ($idAclarador !== "") {
        $modelo->aclarado = new Expression('now()');
      }

      if (!$modelo->save()) {
        $tran->rollBack();
        return (new Respuesta($modelo))
          ->mensaje("Hubo un problema al guardar el registro de la solicitud");
      }

      $solicitud = Solicitud::findOne($idSolicitud);

      $solicitud->aclaracion = $valorAclaracion;

      $bitacora = new BitacoraEstatus();
      $bitacora->uuid();
      $bitacora->idSolicitud = $idSolicitud;
      $bitacora->estatusInicial = $solicitud->idEstatus;
      $bitacora->fechaCambio = new Expression('now()');
      $bitacora->creado = new Expression('now()');
      $bitacora->idUsuario = $usuario->id;

      if ($idSolicitaAclaracion !== "") {
        $estatus = Estatus::find()
          ->andWhere(['nombre' => Estatus::ESTATUS_ACLARACION])
          ->one();
        $bitacora->comentario = $modelo->solicitudAclaracion;
      }

      if ($idAclarador !== "") {
        $estatus = Estatus::find()
          ->andWhere(['nombre' => Estatus::ESTATUS_ACLARADO])
          ->one();
        $bitacora->comentario = $modelo->aclaracion;
      }

      $bitacora->estatusFinal = $estatus->id;
      if (!$bitacora->save()) {
        $tran->rollBack();
        return (new Respuesta($bitacora))
          ->mensaje("Hubo un problema al guardar el registro de la solicitud");
      }

      $bitacora->refresh();

      $solicitud->idEstatus = $estatus->id;

      $solicitud->save();

      foreach ($archivos as $archivo) {
        $media = new AclaracionMedia();

        $media->idMedia = $archivo['id'];
        $media->idAclaracion = $modelo->id;
        $media->tipo = $tipo;
        $media->creado = new Expression('now()');

        if (!$media->save()) {
          $tran->rollBack();
          return (new Respuesta($media))
            ->mensaje("Hubo un problema al guardar el registro de la solicitud");
        }

        $mediaBitacora = new MediaBitacoraEstatus();

        $mediaBitacora->uuid();
        $mediaBitacora->idMedia = $archivo['id'];
        $mediaBitacora->idBitacoraEstatus = $bitacora->id;
        $mediaBitacora->creado = new Expression('now()');

        if (!$mediaBitacora->save()) {
          $tran->rollBack();
          return (new Respuesta($mediaBitacora))
            ->mensaje("Hubo un problema al guardar el registro de la solicitud");
        }
      }

      $tran->commit();
      $modelo->refresh();

      return (new Respuesta($modelo))
        ->mensaje("Registro de solicitud guardado con éxito.");
    } catch (\Exception $e) {
      $tran->rollBack();
      return (new Respuesta())
        ->esError($e)
        ->mensaje("Hubo un error en el servidor");
    }
  }

  public function actionEliminar() {
    $id = intval($this->req->getBodyParam("id", null));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Registro de solicitud no encontrado");
    }

    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar el registro del solicitud");
    }

    return (new Respuesta())
      ->mensaje("Registro de solicitud eliminado");
  }
}
