<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use v1\models\Calendario;
use yii\db\Expression;

class CalendarioController extends AuthController {

	// Establece la clase del modelo a utilizar
	public $modelClass = "v1\models\Calendario";

	// Acción para obtener una lista de calendarios
	public function actionIndex() {
		$id = trim($this->req->get("id", ""));
		$fecha = trim($this->req->get("fecha", ""));

		$query = $this->queryInicial;

		// Filtrar por ID si se proporciona
		if ($id !== "") {
			$query->andWhere(["id" => $id]);
		}

		// Realizar una búsqueda en el año y fecaha del calendario
		if ($fecha !== "") {
			$query->andWhere(["fecha" => $fecha]);
		}

		// Devolver una respuesta con los resultados de la consulta
		return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
	}

	// Acción para guardar un calendario
	public function actionGuardar() {
		$id = trim($this->req->getBodyParam("id", ""));

		$modelo = null;
		if ($id !== "") {
			$modelo = $this->modelClass::findOne($id);
		}

		$tran = \Yii::$app->getDb()->beginTransaction();
		try {
			if ($modelo === null) {
				$modelo = new $this->modelClass();
				$modelo->uuid();
				$modelo->creado = new Expression('now()');
			} else {
				$modelo->modificado = new Expression('now()');
			}

			$modelo->load($this->req->getBodyParams(), "");

			// Guardar el modelo en la base de datos
			if (!$modelo->save()) {
				return (new Respuesta($modelo))
					->mensaje("Hubo un problema al guardar el registro de calendarios");
			}

			$tran->commit();
			$modelo->refresh();

			return (new Respuesta($modelo))
				->mensaje("Registro de calendarios guardado con éxito.");
		} catch (\Exception $e) {
			$tran->rollBack();
			return (new Respuesta())
				->esError($e)
				->mensaje("Hubo un error en el servidor");
		}
	}

	// Acción para eliminar un calendario
	public function actionEliminar() {
		$id = intval($this->req->getBodyParam("id", null));
		$modelo = null;

		if ($id !== "") {
			$modelo = $this->modelClass::findOne(["id" => $id]);
		}
		if ($modelo === null) {
			return (new Respuesta())
				->esError()
				->mensaje("Registro de calendarios no encontrado");
		}

		$modelo->eliminado = new Expression('now()');
		if (!$modelo->save()) {
			return (new Respuesta($modelo))
				->mensaje("No se pudo eliminar el registro de calendarios");
		}

		return (new Respuesta())
			->mensaje("Registro de calendarios eliminado");
	}

	// Acción para calcular una fecha a partir de una fecha dada y una cantidad de días
	public function actionCalcularFecha() {
		$fechaInicio = trim($this->req->get('fecha', ''));
		$dias = intval($this->req->get('dias', ''));

		if ($fechaInicio === '') {
			return (new Respuesta())
				->esError()
				->mensaje('Se necesita una fecha de inicio');
		}

		$fecha = Calendario::calcularDiasHabiles($fechaInicio, $dias);

		return $fecha;
	}
}
