<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class EstatusSolicitudController extends AuthController
{

// Establece la clase del modelo a utilizar
    public $modelClass = "v1\models\EstatusSolicitud";

    public function actionIndex()
    {
        $id = trim($this->req->get("id", ""));
        $buscar = trim($this->req->get("q", ""));
        $notIn = trim($this->req->get("notIn", ""));

        $query = $this->queryInicial;

        if (!empty($id)) {
            $query->andWhere(["id" => $id]);
        }

        if (!empty($notIn)) {
            $explode = explode(",", $notIn);
            $query->andWhere(["NOT IN", "id", $explode]);
        }

        if ($buscar) {
            $query->andWhere([
                "OR",
                "f_unaccent([[nombre]]) ilike f_unaccent(:q)",
                ])->addParams([':q' => "%{$buscar}%"]);
        }

// Devolver una respuesta con los resultados de la consulta
        return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
    }

// Acción para guardar
    public function actionGuardar()
    {
        $id = trim($this->req->getBodyParam("id", ""));
        $modelo = null;

        if (!empty($id)) {
            $modelo = $this->modelClass::findOne($id);
        }
        if ($modelo === null) {
            $modelo = new $this->modelClass();
            $modelo->creado = new Expression('now()');
            $modelo->uuid();
        } else {
            $modelo->modificado = new Expression('now()');
        }

        $modelo->load($this->req->getBodyParams(), '');
// Guardar el modelo en la base de datos
        if (!$modelo->save()) {
            return (new Respuesta($modelo))
                ->mensaje("Hubo un problema al guardar el registro de la solicitud");
        }

        $modelo->refresh();
        return (new Respuesta($modelo))
            ->mensaje("Registro de estatus solicitud guardado con éxito.");
    }

    public function actionEliminar()
    {
        $id = trim($this->req->getBodyParam("id", ""));
        $modelo = null;

        if ($id !== "") {
            $modelo = $this->modelClass::findOne(["id" => $id]);
        }
        if ($modelo === null) {
            return (new Respuesta())
                ->esError()
                ->mensaje("Registro de estatus solicitud no encontrado");
        }
        $modelo->eliminado = new Expression('now()');
        if (!$modelo->save()) {
            return (new Respuesta($modelo))
                ->mensaje("No se pudo eliminar el registro de estatus solicitud");
        }

        return (new Respuesta())
            ->mensaje("Registro de estatus solicitud eliminado");
    }        
}
