<?php

namespace v1\controllers;

use app\models\MensajeMedia;
use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class MensajeController extends AuthController {

  public $modelClass = "v1\models\Mensaje";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $idSolicitud = trim($this->req->get("idSolicitud", ""));
    $buscar = trim($this->req->get("q", ""));

    $query = $this->queryInicial;

    if ($id !== "") {
      $query->andWhere(["id" => $id]);
    }

    if ($idSolicitud !== "") {
      $query->andWhere(["idSolicitud" => $idSolicitud]);
    }

    if ($buscar) {
      $query->andWhere([
        "OR",
        "f_unaccent([[mensaje]]) ilike f_unaccent(:q)",
			])->addParams([':q' => "%{$buscar}%"]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $archivos = $this->req->getBodyParam("archivos", []);

    $modelo = null;
    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }

    $tran = \Yii::$app->getDb()->beginTransaction();
    try {
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo->uuid();
        $modelo->idUsuarioRemitente = $this->usuario->id;
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }

      $modelo->load($this->req->getBodyParams(), '');

      if (!$modelo->save()) {
        return (new Respuesta($modelo))
          ->mensaje("Hubo un problema al guardar el Mensaje");
      }

      $archivosAnteriores = MensajeMedia::find()
        ->where(["idMensaje" => $modelo->id])
        ->all();

      foreach ($archivos as $archivo) {
        $archivoMensaje = null;
        $archivoMensaje = MensajeMedia::find()
          ->andWhere(["idMensaje" => $modelo->id])
          ->andWhere(["idMedia" => $archivo['id']])
          ->one();
        if ($archivoMensaje === null) {
          $archivoMensaje = new MensajeMedia();
          $archivoMensaje->creado = new Expression('now()');
        } else {
          $archivoMensaje->modificado = new Expression('now()');
          if ($archivoMensaje->eliminado) {
            $archivoMensaje->eliminado = null;
          }
        }

        $archivoMensaje->uuid();
        $archivoMensaje->idMensaje = $modelo->id;
        $archivoMensaje->idMedia = $archivo['id'];

        if (!$archivoMensaje->save()) {
          return (new Respuesta($archivoMensaje))
            ->mensaje("Hubo un problema al guardar el archivo");
        }
      }

      foreach ($archivosAnteriores as $archivoAnterior) {
        if (!in_array($archivoAnterior->idMedia, $archivos)) {
          $archivoAnterior->eliminado = new Expression('now()');
          if (!$archivoAnterior->save()) {
            return (new Respuesta($archivoAnterior))
              ->mensaje("Hubo un problema al eliminar el archivo");
          }
        }
      }

      $tran->commit();
      $modelo->refresh();

      return (new Respuesta($modelo))
        ->mensaje("Mensaje guardado con éxito.");
    } catch (\Exception $e) {
      $tran->rollBack();
      return (new Respuesta())
        ->esError($e)
        ->mensaje("Hubo un error en el servidor");
    }
  }

  public function actionEliminar() {
    $id = intval($this->req->getBodyParam("id", null));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Mensaje no encontrado");
    }

    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar el Mensaje");
    }

    return (new Respuesta())
      ->mensaje("Mensaje eliminado");
  }
}
