<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class MunicipioController extends AuthController {

	public $modelClass = "v1\models\Municipio";

	public function actionIndex() {
		$id = trim($this->req->get("id", ""));
		$buscar = trim($this->req->get("q", ""));

		$query = $this->queryInicial;

		if ($id !== "") {
			$query->andWhere(["id" => $id]);
		}

		if ($buscar) {
			$query->andWhere([
				"OR",
				"f_unaccent([[nombre]]) ilike f_unaccent(:q)",
			])->addParams([':q' => "%{$buscar}%"]);
		}

		return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
	}

	public function actionGuardar() {
		$id = trim($this->req->getBodyParam("id", ""));

		$modelo = null;
		if ($id !== "") {
			$modelo = $this->modelClass::findOne($id);
		}

		$tran = \Yii::$app->getDb()->beginTransaction();
		try {
			if ($modelo === null) {
				$modelo = new $this->modelClass();
				$modelo->creado = new Expression('now()');
			} else {
				$modelo->modificado = new Expression('now()');
			}

			if (!$modelo->save()) {
				return (new Respuesta($modelo))
					->mensaje("Hubo un problema al guardar el registro del Municipio");
			}

			$tran->commit();
			$modelo->refresh();

			return (new Respuesta($modelo))
				->mensaje("Registro de Municipio guardado con éxito.");
		} catch (\Exception $e) {
			$tran->rollBack();
			return (new Respuesta())
				->esError($e)
				->mensaje("Hubo un error en el servidor");
		}
	}

	public function actionEliminar() {
		$id = intval($this->req->getBodyParam("id", null));
		$modelo = null;

		if ($id !== "") {
			$modelo = $this->modelClass::findOne(["id" => $id]);
		}
		if ($modelo === null) {
			return (new Respuesta())
				->esError()
				->mensaje("Registro de Municipio no encontrado");
		}

		$modelo->eliminado = new Expression('now()');
		if (!$modelo->save()) {
			return (new Respuesta($modelo))
				->mensaje("No se pudo eliminar el registro del Municipio");
		}

		return (new Respuesta())
			->mensaje("Registro de Municipio eliminado");
	}

}
