<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class PonenciaController extends AuthController {

	public $modelClass = "v1\models\Ponencia";

	public function actionIndex() {
		$id = trim($this->req->get("id", ""));
		$q = trim($this->req->get("q", ""));

		$query = $this->queryInicial;

		if ($id !== "") {
			$query->andWhere(["id" => $id]);
		}

		if ($q) {
			$query->andWhere([
				"OR",
				'f_unaccent([[id]]) ilike f_unaccent(:q)',
				'f_unaccent([[clave]]) ilike f_unaccent(:q)',
				'f_unaccent([[nombre]]) ilike f_unaccent(:q)',
			])->addParams([':q' => "%{$q}%"]);
		}

		return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
	}

	public function actionGuardar() {
		$id = trim($this->req->getBodyParam("id", null));
		$modelo = null;

		if ($id !== "") {
			$modelo = $this->modelClass::findOne($id);
		}

		try {
			if ($modelo === null) {
				$modelo = new $this->modelClass();
				$modelo->uuid();
				$modelo->creado = new Expression('now()');
			} else {
				$modelo->modificado = new Expression('now()');
			}

			$modelo->load($this->req->getBodyParams(), '');
			if (!$modelo->save()) {
				return (new Respuesta($modelo))
					->mensaje("Hubo un problema al guardar la Ponencia");
			}

			$modelo->refresh();
			return (new Respuesta($modelo))
				->mensaje("Registro de Ponencia guardado");
		} catch (\Exception $e) {
			return (new Respuesta())
				->esError()
				->mensaje("Hubo un error en el servidor" . $e->getMessage());
		}
	}

	public function actionEliminar() {
		$id = trim($this->req->getBodyParam("id", null));
		$modelo = null;

		if ($id > 0) {
			$modelo = $this->modelClass::findOne(["id" => $id]);
		}
		if ($modelo === null) {
			return (new Respuesta())
				->esError()
				->mensaje("Registro de Ponencia no encontrada");
		}
		$modelo->eliminado = new Expression('now()');
		if (!$modelo->save()) {
			return (new Respuesta($modelo))
				->mensaje("No se pudo eliminar el registro de Ponencia");
		}

		return (new Respuesta())
			->mensaje("Ponencia eliminado");
	}
}
