<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use v1\models\Permiso;
use yii\db\Expression;

class SujetoObligadoController extends AuthController {

  public $modelClass = "v1\models\SujetoObligado";
  public $permisos = [
    Permiso::SUJETO_OBLIGADO_VER_TODO
  ];

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $idCategoria = trim($this->req->get("idCategoria", ""));
    $incompetencia = intval($this->req->get('incompetencia', ''));
    $buscar = trim($this->req->get("q", ""));

    $usuario = $this->usuario;

    $query = $this->queryInicial;

    if ($incompetencia < 1 && !$usuario->tienePermiso(Permiso::SUJETO_OBLIGADO_VER_TODO)) {
      $query->andWhere(['{{SujetoObligado}}.[[id]]' => $usuario->idSujetoObligado]);
    }

    if ($id !== "") {
      $query->andWhere(["id" => $id]);
    }

    if ($idCategoria !== "") {
      $query->andWhere(["idCategoria" => $idCategoria]);
    }

    if ($buscar) {
      $query->andWhere([
        "OR",
        "f_unaccent([[nombre]]) ilike f_unaccent(:q)",
        "f_unaccent([[sigla]]) ilike f_unaccent(:q)",
      ])->addParams([':q' => "%{$buscar}%"]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));

    $modelo = null;
    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }

    $tran = \Yii::$app->getDb()->beginTransaction();
    try {
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }

      $modelo->load($this->req->getBodyParams(), "");

      if ($modelo->isNewRecord) {
        $modelo->secuencialSolicitud = 1;
      }

      if (!$modelo->save()) {
        $tran->rollBack();
        return (new Respuesta($modelo))
          ->mensaje("Hubo un problema al guardar el registro del sujeto obligado");
      }

      $tran->commit();
      $modelo->refresh();

      return (new Respuesta($modelo))
        ->mensaje("Registro de sujeto obligado guardado con éxito.");
    } catch (\Exception $e) {
      $tran->rollBack();
      return (new Respuesta())
        ->esError($e)
        ->mensaje("Hubo un error en el servidor");
    }
  }

  public function actionEliminar() {
    $id = intval($this->req->getBodyParam("id", null));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Registro de sujeto obligado no encontrado");
    }

    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar el registro del sujeto obligado");
    }

    return (new Respuesta())
      ->mensaje("Registro de sujeto obligado eliminado");
  }
}
