<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class TemaSolicitudController extends AuthController {

  // Establece la clase del modelo a utilizar
  public $modelClass = "v1\models\TemaSolicitud";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $notIn = trim($this->req->get("notIn", ""));

    $query = $this->queryInicial;

    if (!empty($id)) {
      $query->andWhere(["id" => $id]);
    }

    if (!empty($notIn)) {
      $explode = explode(",", $notIn);
      $query->andWhere(["NOT IN", "id", $explode]);
    }

    if ($buscar) {
      $query->andWhere([
        "OR",
        "f_unaccent([[nombre]]) ilike f_unaccent(:q)",
      ])->addParams([':q' => "%{$buscar}%"]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  // Acción para guardar un calendario
  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));

    $modelo = null;

    if (!empty($id)) {
      $modelo = $this->modelClass::findOne($id);
    }
    if ($modelo === null) {
      $modelo = new $this->modelClass();
      $modelo->creado = new Expression('now()');
      $modelo->uuid();
    } else {
      $modelo->modificado = new Expression('now()');
    }

    $modelo->load($this->req->getBodyParams(), '');
    // Guardar el modelo en la base de datos
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("Hubo un problema al guardar el registro de Solicitud");
    }

    $modelo->refresh();
    return (new Respuesta($modelo))
      ->mensaje("Registro de Solicitud guardado con éxito.");
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Registro de Solicitud no encontrado");
    }
    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar el registro de Solicitud");
    }

    return (new Respuesta())
      ->mensaje("Registro de Solicitud eliminado");
  }
}
