<?php

namespace v1\models;

use app\models\Solicitud as ModelsSolicitud;

class Solicitud extends ModelsSolicitud {

  public function fields() {
    return [
      'id',
      'folio',
      'codigoSolicitante',
      'descripcion',
      'archivoAnexo',
      'idModalidadEntrega',
      'otroModalidadEntrega',
      'tipoSolicitud',
      'idSujetoObligado',
      'consecutivoSolicitudesSujetoObligado',
      'aclaracion',
      'tipoCambioEstado',
      'recursoRevision',
      'lugarRecepcion',
      'tipoAccesoSolicitud',
      'idEstatus',
      'idUsuario',
      'idParticular',
      'banderaEstadoSolicitud',
      'tipoPreSolicitud',
      'alta',
      'recepcionSistema',
      'ipSolicita',
      'recepcion',
      'sexo',
      'edad',
      'ocupacion',
      'medioNotificacion',
      'representacion',
      'documento',
      'otro',
      'educacion',
      'otroEducacion',
      'derechoAcc',
      'otroDerechoAcc',
      'otroOcupacion',
      'nombre',
      'primerApellido',
      'segundoApellido',
      'tipoPersona',
      'razonSocial',
      'domicilioExt',
      'pais',
      'estado',
      'ciudad',
      'municipio',
      'colonia',
      'calle',
      'numeroExterior',
      'numeroInterior',
      'codigoPostal',
      'correo',
      'repLegal',
      'nombreRepLegal',
      'telefono',
      'coloniaExt',
      'estadoExt',
      'sistema',
      'medidasAccesibilidad',
      'otraMedidaAcceso',
      'lenguaIndigena',
      'entidad',
      'idMedioRecepcion',
      'correoMedioRecepcion',
      'idFormatoAcceso',
      'puebloIndigena',
      'nacionalidad',
      'idTipoDerecho',
      'idSujetoObligadoRec',
      'limiteRespuesta',
      'ocupaciones',
      'accesibilidad',
      'municipioExt',
      'tema',
      'subtema',
      'otroTema',
      'otroSubtema',
      'creado',
      'modificado',
      'eliminado',
      'idTema',
      'idSubtema',
      'idSujetoObligadoInicial',
      'motivoDeclinacion',
      'declinado',
      'idUsuarioDeclina',
      'idSubestatus',
      'diasTranscurridos',
      'diasRestantes',
      'diasAtencion',
      'idSolicitudDerivada',
    ];
  }

  public function extraFields() {
    return [
      'bitacorasEstatus',
      'ultimaBitacora',
      'estatus',
      'particular',
      'solicitudDerivada',
      'sujetoObligado',
      'sujetoObligadoInicial',
      'usuario',
      'usuarioDeclina',
      'solicitudExpedientes',
      'solicitudMedia',
      'media',
      'recursosRevision',
      'subtemaSolicitud',
      'temaSolicitud',
      'solicitudOriginal',
    ];
  }

  public function getBitacorasEstatus() {
    return $this->hasMany(BitacoraEstatus::class, ['idSolicitud' => 'id']);
  }

  public function getUltimaBitacora() {
    return $this->hasOne(BitacoraEstatus::class, ['idSolicitud' => 'id'])->orderBy(['creado' => SORT_DESC]);
  }

  public function getEstatus() {
    return $this->hasOne(Estatus::class, ['id' => 'idEstatus']);
  }

  public function getParticular() {
    return $this->hasOne(Particular::class, ['id' => 'idParticular']);
  }

  public function getSolicitudDerivada() {
    return $this->hasOne(Solicitud::class, ['id' => 'idSolicitudDerivada']);
  }

  public function getSubtemaSolicitud() {
    return $this->hasOne(SubtemaSolicitud::class, ['id' => 'idSubtema']);
  }

  public function getSujetoObligado() {
    return $this->hasOne(SujetoObligado::class, ['id' => 'idSujetoObligado']);
  }

  public function getSujetoObligadoInicial() {
    return $this->hasOne(SujetoObligado::class, ['id' => 'idSujetoObligadoInicial']);
  }

  public function getTemaSolicitud() {
    return $this->hasOne(TemaSolicitud::class, ['id' => 'idTema']);
  }

  public function getUsuario() {
    return $this->hasOne(Usuario::class, ['id' => 'idUsuario']);
  }

  public function getUsuarioDeclina() {
    return $this->hasOne(Usuario::class, ['id' => 'idUsuarioDeclina']);
  }

  public function getSolicitudExpedientes() {
    return $this->hasMany(SolicitudExpediente::class, ['idSolicitud' => 'id']);
  }

  public function getSolicitudMedia() {
    return $this->hasMany(SolicitudMedia::class, ['idSolicitud' => 'id']);
  }

  public function getMedia() {
    return $this->hasMany(Media::class, ['id' => 'idMedia'])
      ->viaTable('SolicitudMedia', ['idSolicitud' => 'id']);
  }

  public function getRecursosRevision() {
    return $this->hasMany(RecursoRevision::class, ['idSolicitud' => 'id']);
  }

  public function getSolicitudOriginal() {
    return $this->hasOne(Solicitud::class, ['idSolicitudDerivada' => 'id']);
  }
}
